"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StateProcessor = void 0;
const jsonpath_plus_1 = require("jsonpath-plus");
const LambdaWaitFotTokenPayloadTemplate_1 = require("./PayloadTemplates/LambdaWaitFotTokenPayloadTemplate");
const ParameterPayloadTemplate_1 = require("./PayloadTemplates/ParameterPayloadTemplate");
const Logger_1 = require("./utils/Logger");
class StateProcessor {
    static processInputPath(dataJson, inputPath) {
        this.logger.debug(`StateProcessor - processInputPath - ${dataJson}`);
        if (inputPath === null) {
            return '{}';
        }
        const inputJson = dataJson || '{}';
        const result = jsonpath_plus_1.JSONPath({
            path: inputPath === undefined ? '$' : inputPath,
            json: JSON.parse(inputJson),
        });
        if (!result || result.length === 0) {
            throw new Error('');
        }
        return JSON.stringify(result[0]);
    }
    static processWaitForTokenParameters(dataJson, parameters, context) {
        const inputJson = dataJson || '{}';
        const payloadTemplate = LambdaWaitFotTokenPayloadTemplate_1.LambdaWaitFotTokenPayloadTemplate.create(parameters, context);
        return JSON.stringify(payloadTemplate.process(inputJson));
    }
    static processParameters(dataJson, payloadTemplateInput) {
        if (!payloadTemplateInput) {
            return dataJson || '{}';
        }
        const inputJson = dataJson || '{}';
        const payloadTemplate = ParameterPayloadTemplate_1.ParameterPayloadTemplate.create(payloadTemplateInput);
        return JSON.stringify(payloadTemplate.process(inputJson));
    }
    static processResultPath(input, result, resultPath) {
        if (!resultPath || resultPath === '$') {
            return JSON.stringify(result);
        }
        const resultPathArray = jsonpath_plus_1.JSONPath.toPathArray(resultPath);
        let temp = input;
        for (let i = 1; i < resultPathArray.length; i++) {
            const key = resultPathArray[i];
            if (i === resultPathArray.length - 1) {
                temp[key] = result;
            }
            else {
                if (!temp[key]) {
                    temp[key] = {};
                }
                temp = temp[key];
            }
        }
        return JSON.stringify(input);
    }
    static processOutputPath(json, outputPath) {
        if (!json) {
            throw new Error('Output JSON of lambda was undefined');
        }
        const result = jsonpath_plus_1.JSONPath({
            path: outputPath || '$',
            json: JSON.parse(json),
        });
        if (!result || result.length === 0) {
            throw new Error('');
        }
        return JSON.stringify(result[0]);
    }
}
exports.StateProcessor = StateProcessor;
StateProcessor.logger = Logger_1.Logger.getInstance();
