"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapExecutor = void 0;
const StateTypeExecutor_1 = require("../StateTypeExecutor");
const StateMachineExecutor_1 = require("../../StateMachineExecutor");
const StateContext_1 = require("../../Context/StateContext");
const StateProcessor_1 = require("../../StateProcessor");
const StateMachine_1 = require("../../StateMachine/StateMachine");
class MapExecutor extends StateTypeExecutor_1.StateTypeExecutor {
    constructor() {
        super(...arguments);
        this.pendingStateMachineExecutions = {};
    }
    async execute(context, stateDefinition, inputJson) {
        const stateMachineDescription = {
            name: stateDefinition.Comment,
            definition: stateDefinition.Iterator,
        };
        const stateMachine = StateMachine_1.StateMachine.create(stateMachineDescription.name || '', stateMachineDescription);
        if (!inputJson) {
            throw new Error(`Undefined inputJson for state `);
        }
        const iterable = this.processInput(inputJson, stateDefinition, context);
        if (!Array.isArray(iterable)) {
            this.logger.error(`Processed input is not an array with InputPath: ${stateDefinition.InputPath} and input: ${inputJson}`);
            throw new Error('Input is not an array');
        }
        const output = [];
        await Promise.all(iterable.map(async (value) => {
            const tempContext = context.clone();
            const stateName = stateDefinition.Iterator.StartAt;
            const stateContext = StateContext_1.StateContext.create(stateName);
            tempContext.transitionTo(stateContext);
            const sme = new StateMachineExecutor_1.StateMachineExecutor(stateMachine, tempContext);
            const startAtState = stateDefinition.Iterator.States[stateDefinition.Iterator.StartAt];
            output.push(await sme.execute(startAtState, JSON.stringify(value)));
        }));
        return {
            Next: stateDefinition.Next,
            End: stateDefinition.End,
            json: JSON.stringify(output),
        };
    }
    processInput(json, stateDefinition, context) {
        this.logger.debug(`MapExecutor - processInput1 - ${json}`);
        const proccessedInputJson = StateProcessor_1.StateProcessor.processInputPath(json, stateDefinition.ItemsPath);
        this.logger.debug(`MapExecutor - processInput2 - ${proccessedInputJson}`);
        try {
            return JSON.parse(proccessedInputJson);
        }
        catch (error) {
            this.logger.error(`MapExecutor.processInput: Could not parse JSON for state ${context.State.Name}: "${proccessedInputJson}"`);
            throw error;
        }
    }
}
exports.MapExecutor = MapExecutor;
